/*  TWI (I2C) code to drive 128x64 monochrome oled display modules
 *  ceptimus.  September 2016
 */
#ifndef SX1X06_h
#define SX1X06_h

#include "Arduino.h"
#include "ssdfont.h"

// #define SH1106 // comment out one or other of these lines so as to leave the definition active for the type of display driver you have
#define SSD1306 // comment out one or other of these lines so as to leave the definition active for the type of display driver you have

#define SX1X06_I2C_ADDRESS 0x3C // some modules can be 0x3D - depends on how module's internal SA0 has been wired


// pixels
#define SX1X06_LCDWIDTH  128
#define SX1X06_LCDHEIGHT  64

// display chars assuming 6x8 font
#define SX1X06_CHARWIDTH (SX1X06_LCDWIDTH / 6)
#define SX1X06_CHARHEIGHT (SX1X06_LCDHEIGHT / 8)
#define SX1X06_NUMCHARS (SX1X06_CHARWIDTH * SX1X06_CHARHEIGHT)

// there may be unused pixels down each side of the display - for example with a 128 pixel-width display and a 6-bit-wide font there are 21 characters per row with 1 unused pixel down each edge
#define SX1X06_LEFTMARGIN ((SX1X06_LCDWIDTH - SX1X06_CHARWIDTH * 6) / 2)  
// (unlikely to need a TOPMARGIN as displays are all likely to be a multiple of 8 pixels high - in any case we'll manage okay without)


// give names to the command bytes the display module understands - these are mostly just for the SSD1306 as support for the SH1106 was added later - see datasheets.
#define SX1X06_SETCONTRAST 0x81
#define SX1X06_DISPLAYALLON_RESUME 0xA4
#define SX1X06_DISPLAYALLON 0xA5
#define SX1X06_NORMALDISPLAY 0xA6
#define SX1X06_INVERTDISPLAY 0xA7
#define SX1X06_DISPLAYOFF 0xAE
#define SX1X06_DISPLAYON 0xAF

#define SX1X06_SETDISPLAYOFFSET 0xD3
#define SX1X06_SETCOMPINS 0xDA

#define SX1X06_SETVCOMDETECT 0xDB

#define SX1X06_SETDISPLAYCLOCKDIV 0xD5
#define SX1X06_SETPRECHARGE 0xD9

#define SX1X06_SETMULTIPLEX 0xA8

#define SX1X06_SETLOWCOLUMN 0x00
#define SX1X06_SETHIGHCOLUMN 0x10

#define SX1X06_COLUMNADDR 0x21
#define SX1X06_PAGEADDR   0x22

#define SX1X06_SETSTARTLINE 0x40
#define SX1X06_SETSTARTPAGE 0XB0
#define SX1X06_MEMORYMODE 0x20

#define SX1X06_COMSCANINC 0xC0
#define SX1X06_COMSCANDEC 0xC8

#define SX1X06_SEGREMAP 0xA0

#define SX1X06_CHARGEPUMP 0x8D

#define SX1X06_EXTERNALVCC 0x1
#define SX1X06_SWITCHCAPVCC 0x2

class SX1X06 {
  public:
    static void start(void);
    static void fillDisplay(uint8_t c); // fill whole display with (c) characters.  c is most likely ' ' or (' ' | 0x80) for reversed field
    static void displayString6x8(uint8_t row, uint8_t col, const char *s, uint8_t rvsField); // if rvsField != 0 display characters in string with reversed field
    static void displayString6x8(uint8_t row, uint8_t col, const __FlashStringHelper *s, uint8_t rvsField); // same as above for constant (FLASH memory) stored strings
    static void drawImage(uint8_t *img, uint8_t x, uint8_t row, uint8_t width, uint8_t rows);
  private:
    static void displayChar6x8(uint8_t row, uint8_t x, uint8_t c);
};

#endif

